@@ -157,7 +157,8 @@ MVMObject * MVM_sc_get_object(MVMThreadContext *tc, MVMSerializationContext *sc,
         return roots[idx] ? roots[idx] : MVM_serialization_demand_object(tc, sc, idx);
     else
         MVM_exception_throw_adhoc(tc,
-            "No object at index %d", idx);
+            "Probable version skew in pre-compiled '%s' (cause: no object at index %"PRId64")",
+            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
 }
 
 MVMObject * MVM_sc_get_sc_object(MVMThreadContext *tc, MVMCompUnit *cu,
@@ -186,10 +187,8 @@ MVMObject * MVM_sc_try_get_object(MVMThreadContext *tc, MVMSerializationContext
 
 /* Given an SC, an index, and an object, store the object at that index. */
 void MVM_sc_set_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *obj) {
-    MVMObject **roots = sc->body->root_objects;
-    MVMint64    count = sc->body->num_objects;
     if (idx < 0)
-        MVM_exception_throw_adhoc(tc, "Invalid (negative) object root index %d", idx);
+        MVM_exception_throw_adhoc(tc, "Invalid (negative) object root index %"PRId64"", idx);
     if (idx < sc->body->num_objects) {
         /* Just updating an existing one. */
         MVM_ASSIGN_REF(tc, &(sc->common.header), sc->body->root_objects[idx], obj);
@@ -217,7 +216,9 @@ MVMSTable * MVM_sc_get_stable(MVMThreadContext *tc, MVMSerializationContext *sc,
         return got ? got : MVM_serialization_demand_stable(tc, sc, idx);
     }
     else {
-        MVM_exception_throw_adhoc(tc, "No STable at index %d", idx);
+        MVM_exception_throw_adhoc(tc,
+            "Probable version skew in pre-compiled '%s' (cause: no STable at index %"PRId64")",
+            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
     }
 }
 
@@ -234,7 +235,7 @@ MVMSTable * MVM_sc_try_get_stable(MVMThreadContext *tc, MVMSerializationContext
 void MVM_sc_set_stable(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMSTable *st) {
     if (idx < 0)
         MVM_exception_throw_adhoc(tc,
-            "Invalid (negative) STable index", idx);
+            "Invalid (negative) STable index %"PRId64, idx);
     if (idx < sc->body->num_stables) {
         /* Just updating an existing one. */
         MVM_ASSIGN_REF(tc, &(sc->common.header), sc->body->root_stables[idx], st);
@@ -280,7 +281,8 @@ MVMObject * MVM_sc_get_code(MVMThreadContext *tc, MVMSerializationContext *sc, M
     }
     else {
         MVM_exception_throw_adhoc(tc,
-            "No code ref at index %d", idx);
+            "Probable version skew in pre-compiled '%s' (cause: no code ref at index %"PRId64")",
+            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
     }
 }
 
