@@ -377,6 +377,8 @@ MVMObject * MVM_exception_backtrace(MVMThreadContext *tc, MVMObject *ex_obj) {
         MVM_repr_bind_key_o(tc, row, k_anno, annotations);
 
         MVM_repr_push_o(tc, arr, row);
+        MVM_free(annot);
+
         cur_frame = cur_frame->caller;
         while (cur_frame && cur_frame->static_info->body.is_thunk)
             cur_frame = cur_frame->caller;
@@ -681,7 +683,7 @@ void MVM_panic(MVMint32 exitCode, const char *messageFormat, ...) {
 
 MVM_NO_RETURN
 void MVM_panic_allocation_failed(size_t len) {
-    MVM_panic(1, "Memory allocation failed; could not allocate %zu bytes", len);
+    MVM_panic(1, "Memory allocation failed; could not allocate %"MVM_PRSz" bytes", len);
 }
 
 /* Throws an ad-hoc (untyped) exception. */
