@@ -19,7 +19,7 @@ static void check_strand_sanity(MVMThreadContext *tc, MVMString *s) {
 }
 #define STRAND_CHECK(tc, s) check_strand_sanity(tc, s);
 #else
-#define STRAND_CHECK(tc, s) 0
+#define STRAND_CHECK(tc, s)
 #endif
 
 /* Allocates strand storage. */
@@ -100,7 +100,6 @@ MVMint64 MVM_string_substrings_equal_nocheck(MVMThreadContext *tc, MVMString *a,
 
 /* Returns the codepoint without doing checks, for internal VM use only. */
 MVMGrapheme32 MVM_string_get_grapheme_at_nocheck(MVMThreadContext *tc, MVMString *a, MVMint64 index) {
-    MVMStringIndex idx = (MVMStringIndex)index;
     switch (a->body.storage_type) {
     case MVM_STRING_GRAPHEME_32:
         return a->body.storage.blob_32[index];
@@ -220,7 +219,7 @@ MVMString * MVM_string_substring(MVMThreadContext *tc, MVMString *a, MVMint64 of
 
     /* -1 signifies go to the end of the string; anything less is a bug */
     if (length < -1)
-        MVM_exception_throw_adhoc(tc, "Substring length (%lld) cannot be negative", length);
+        MVM_exception_throw_adhoc(tc, "Substring length (%"PRId64") cannot be negative", length);
 
     /* negative offsets count from the end */
     start_pos = offset < 0 ? offset + agraphs : offset;
@@ -233,7 +232,7 @@ MVMString * MVM_string_substring(MVMThreadContext *tc, MVMString *a, MVMint64 of
     }
 
     if (end_pos < 0)
-        MVM_exception_throw_adhoc(tc, "Substring end (%lld) cannot be less than 0", end_pos);
+        MVM_exception_throw_adhoc(tc, "Substring end (%"PRId64") cannot be less than 0", end_pos);
 
     /* Ensure we're within bounds. */
     if (start_pos < 0)
@@ -427,9 +426,9 @@ MVMString * MVM_string_repeat(MVMThreadContext *tc, MVMString *a, MVMint64 count
     if (count == 1)
         return a;
     if (count < 0)
-        MVM_exception_throw_adhoc(tc, "repeat count (%lld) cannot be negative", count);
+        MVM_exception_throw_adhoc(tc, "repeat count (%"PRId64") cannot be negative", count);
     if (count > (1 << 30))
-        MVM_exception_throw_adhoc(tc, "repeat count > %lld arbitrarily unsupported...", (1 << 30));
+        MVM_exception_throw_adhoc(tc, "repeat count > %d arbitrarily unsupported...", (1 << 30));
 
     /* If input string is empty, repeating it is empty. */
     agraphs = MVM_string_graphs(tc, a);
@@ -524,10 +523,13 @@ MVMint64 MVM_string_equal_at(MVMThreadContext *tc, MVMString *a, MVMString *b, M
 }
 
 MVMint64 MVM_string_equal_at_ignore_case(MVMThreadContext *tc, MVMString *a, MVMString *b, MVMint64 offset) {
-    MVMString *lca = MVM_string_lc(tc, a);
+    MVMString *lca;
     MVMString *lcb;
-    MVMROOT(tc, lca, {
-        lcb = MVM_string_lc(tc, b);
+    MVMROOT(tc, b, {
+        lca = MVM_string_lc(tc, a);
+        MVMROOT(tc, lca, {
+            lcb = MVM_string_lc(tc, b);
+        });
     });
     return MVM_string_equal_at(tc, lca, lcb, offset);
 }
@@ -570,7 +572,7 @@ MVMint64 MVM_string_get_grapheme_at(MVMThreadContext *tc, MVMString *a, MVMint64
     agraphs = MVM_string_graphs(tc, a);
 
     if (index < 0 || index >= agraphs)
-        MVM_exception_throw_adhoc(tc, "Invalid string index: max %lld, got %lld",
+        MVM_exception_throw_adhoc(tc, "Invalid string index: max %"PRId32", got %"PRId64,
             agraphs - 1, index);
 
     return (MVMint64)MVM_string_get_grapheme_at_nocheck(tc, a, index);
@@ -614,7 +616,9 @@ MVMString * funcname(MVMThreadContext *tc, MVMString *s) { \
         MVM_string_gi_init(tc, &gi, s); \
         while (i < sgraphs) { \
             MVMGrapheme32 before = MVM_string_gi_get_grapheme(tc, &gi); \
-            MVMGrapheme32 after  = MVM_unicode_get_case_change(tc, before, type); \
+            MVMGrapheme32 after  = before >= 0 \
+                ? MVM_unicode_get_case_change(tc, before, type) \
+                : MVM_nfg_get_case_change(tc, before, type); \
             result_buf[i++]      = after; \
             if (before != after) \
                 changed = 1; \
@@ -652,7 +656,7 @@ MVMString * MVM_string_decode(MVMThreadContext *tc,
         case MVM_encoding_type_windows1252:
             return MVM_string_windows1252_decode(tc, type_object, Cbuf, byte_length);
         default:
-            MVM_exception_throw_adhoc(tc, "invalid encoding type flag: %d", encoding_flag);
+            MVM_exception_throw_adhoc(tc, "invalid encoding type flag: %"PRId64, encoding_flag);
     }
     return NULL;
 }
@@ -671,7 +675,7 @@ char * MVM_string_encode(MVMThreadContext *tc, MVMString *s, MVMint64 start, MVM
         case MVM_encoding_type_windows1252:
             return MVM_string_windows1252_encode_substr(tc, s, output_size, start, length);
         default:
-            MVM_exception_throw_adhoc(tc, "invalid encoding type flag: %d", encoding_flag);
+            MVM_exception_throw_adhoc(tc, "invalid encoding type flag: %"PRId64, encoding_flag);
     }
     return NULL;
 }
@@ -1002,15 +1006,21 @@ MVMint64 MVM_string_char_at_in_string(MVMThreadContext *tc, MVMString *a, MVMint
 }
 
 MVMint64 MVM_string_offset_has_unicode_property_value(MVMThreadContext *tc, MVMString *s, MVMint64 offset, MVMint64 property_code, MVMint64 property_value_code) {
+    MVMGrapheme32 g;
+    MVMCodepoint  cp;
+
     if (!IS_CONCRETE((MVMObject *)s)) {
         MVM_exception_throw_adhoc(tc, "uniprop lookup needs a concrete string");
     }
-
     if (offset < 0 || offset >= MVM_string_graphs(tc, s))
         return 0;
 
-    return MVM_unicode_codepoint_has_property_value(tc,
-        MVM_string_get_grapheme_at_nocheck(tc, s, offset), property_code, property_value_code);
+    g = MVM_string_get_grapheme_at_nocheck(tc, s, offset);
+    if (g >= 0)
+        cp = (MVMCodepoint)g;
+    else
+        cp = MVM_nfg_get_synthetic_info(tc, g)->base;
+    return MVM_unicode_codepoint_has_property_value(tc, cp, property_code, property_value_code);
 }
 
 /* Normalizes a string to a flat MVMGrapheme32 buffer, for the benefit of
@@ -1146,14 +1156,14 @@ MVMint64 MVM_string_compare(MVMThreadContext *tc, MVMString *a, MVMString *b) {
                           0 ;
 }
 
-/* Takes two strings and AND's their charaters. */
+/* Takes two strings and AND's their characters. */
 MVMString * MVM_string_bitand(MVMThreadContext *tc, MVMString *a, MVMString *b) {
     MVMString         *res = NULL;
     MVMStringIndex    alen = MVM_string_graphs(tc, a);
     MVMStringIndex    blen = MVM_string_graphs(tc, b);
     MVMStringIndex sgraphs = alen < blen ? alen : blen;
     MVMGrapheme32  *buffer = MVM_malloc(sizeof(MVMGrapheme32) * sgraphs);
-    MVMStringIndex i, scanlen;
+    MVMStringIndex i;
 
     /* Binary-and up to the length of the shortest string. */
     for (i = 0; i < sgraphs; i++)
@@ -1169,7 +1179,7 @@ MVMString * MVM_string_bitand(MVMThreadContext *tc, MVMString *a, MVMString *b)
     return res;
 }
 
-/* Takes two strings and OR's their charaters. */
+/* Takes two strings and OR's their characters. */
 MVMString * MVM_string_bitor(MVMThreadContext *tc, MVMString *a, MVMString *b) {
     MVMString         *res = NULL;
     MVMStringIndex    alen = MVM_string_graphs(tc, a);
@@ -1201,7 +1211,7 @@ MVMString * MVM_string_bitor(MVMThreadContext *tc, MVMString *a, MVMString *b) {
     return res;
 }
 
-/* Takes two strings and XOR's their charaters. */
+/* Takes two strings and XOR's their characters. */
 MVMString * MVM_string_bitxor(MVMThreadContext *tc, MVMString *a, MVMString *b) {
     MVMString         *res = NULL;
     MVMStringIndex    alen = MVM_string_graphs(tc, a);
@@ -1301,9 +1311,13 @@ void MVM_string_cclass_init(MVMThreadContext *tc) {
 }
 
 /* Checks if the specified grapheme is in the given character class. */
-static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGrapheme32 cp) {
-    if (cp < 0)
-        MVM_exception_throw_adhoc(tc, "Negative character fed to cclass: '%d'", cp);
+static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGrapheme32 g) {
+    /* If it's a synthetic, then grab the base codepoint. */
+    MVMCodepoint cp;
+    if (g >= 0)
+        cp = (MVMCodepoint)g;
+    else
+        cp = MVM_nfg_get_synthetic_info(tc, g)->base;
 
     switch (cclass) {
         case MVM_CCLASS_ANY:
@@ -1319,16 +1333,16 @@ static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGra
 
         case MVM_CCLASS_WORD:
             if (cp <= 'z') {  /* short circuit common case */
-		if (cp >= 'a' || cp == '_' || (cp >= 'A' && cp <= 'Z') || (cp >= '0' && cp <= '9'))
-		    return 1;
-		else
-		    return 0;
-	    }
+                if (cp >= 'a' || cp == '_' || (cp >= 'A' && cp <= 'Z') || (cp >= '0' && cp <= '9'))
+                    return 1;
+                else
+                    return 0;
+            }
             /* Deliberate fall-through; word is _ or digit or alphabetic. */
 
         case MVM_CCLASS_ALPHANUMERIC:
-	    if (cp <= '9' && cp >= '0')  /* short circuit common case */
-		return 1;
+            if (cp <= '9' && cp >= '0')  /* short circuit common case */
+                return 1;
             if (MVM_unicode_codepoint_has_property_value(tc, cp,
                     MVM_UNICODE_PROPERTY_GENERAL_CATEGORY, UPV_Nd))
                 return 1;
@@ -1336,11 +1350,11 @@ static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGra
 
         case MVM_CCLASS_ALPHABETIC:
             if (cp <= 'z') {  /* short circuit common case */
-		if (cp >= 'a' || (cp >= 'A' && cp <= 'Z'))
-		    return 1;
-		else
-		    return 0;
-	    }
+                if (cp >= 'a' || (cp >= 'A' && cp <= 'Z'))
+                    return 1;
+                else
+                    return 0;
+            }
             return
                 MVM_unicode_codepoint_has_property_value(tc, cp,
                     MVM_UNICODE_PROPERTY_GENERAL_CATEGORY, UPV_Lo) /* lots of CJK chars */
@@ -1354,8 +1368,8 @@ static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGra
                     MVM_UNICODE_PROPERTY_GENERAL_CATEGORY, UPV_Lm);
 
         case MVM_CCLASS_NUMERIC:
-	    if (cp <= '9' && cp >= '0')  /* short circuit common case */
-		return 1;
+            if (cp <= '9' && cp >= '0')  /* short circuit common case */
+                return 1;
             return MVM_unicode_codepoint_has_property_value(tc, cp,
                 MVM_UNICODE_PROPERTY_GENERAL_CATEGORY, UPV_Nd);
 
@@ -1364,12 +1378,12 @@ static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGra
                 MVM_UNICODE_PROPERTY_ASCII_HEX_DIGIT, 1);
 
         case MVM_CCLASS_WHITESPACE:
-	    if (cp <= '~') {
-		if (cp == ' ' || (cp <= 13 && cp >= 9))
-		    return 1;
-		else
-		    return 0;
-	    }
+            if (cp <= '~') {
+                if (cp == ' ' || (cp <= 13 && cp >= 9))
+                    return 1;
+                else
+                    return 0;
+            }
             return MVM_unicode_codepoint_has_property_value(tc, cp,
                 MVM_UNICODE_PROPERTY_WHITE_SPACE, 1);
 
@@ -1420,11 +1434,11 @@ static MVMint64 grapheme_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMGra
 /* Checks if the character at the specified offset is a member of the
  * indicated character class. */
 MVMint64 MVM_string_is_cclass(MVMThreadContext *tc, MVMint64 cclass, MVMString *s, MVMint64 offset) {
-    MVMGrapheme32 cp;
+    MVMGrapheme32 g;
     if (offset < 0 || offset >= MVM_string_graphs(tc, s))
         return 0;
-    cp = MVM_string_get_grapheme_at_nocheck(tc, s, offset);
-    return grapheme_is_cclass(tc, cclass, cp);
+    g = MVM_string_get_grapheme_at_nocheck(tc, s, offset);
+    return grapheme_is_cclass(tc, cclass, g);
 }
 
 /* Searches for the next char that is in the specified character class. */
